﻿/**************************************************************************
 *
 * Filename: Program.cs   
 *
 * Copyright: Alphons van der Heijden 2012
 *
 * Author: AAB van der Heijden
 *
 * Description:
 *   This is a console-mode program that demonstrates how to use the
 *   PicoLog 1000 series driver in C# .NET
 *
 * There are the following examples:
 *    Collect a block of samples immediately
 *    Collect a block of samples when a trigger event occurs
 *    Use windowing to collect a sequence of overlapped blocks
 *    Write a continuous stream of data to a disk file
 *    Take individual readings
 *		Set PWM
 *		Set digital outputs
 *
 * To build this application
 *	place PL1000.dll in the same directory as the .NET executable,
 *	place PicoLogNet.dll in the same directory as the .NET executable
 *	and build it
 *
 ***************************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.IO;
using System.Threading;

using PicoLogNet;

namespace PicoLogTestConsole
{
	class Program
	{
		static short g_handle;
		static Pico.StatusEnum status;

		static short d0State, d1State, d2State, d3State = 0;

		static bool scale_to_mv = true;
		static short max_adc_value = 0;

		/****************************************************************************
		 *
		 * adc_to_mv
		 *
		 * If the user selects scaling to millivolts,
		 * Convert a 10- or 12-bit ADC count into millivolts
		 *
		 ****************************************************************************/
		static int adc_to_mv(int raw)
		{
			int scaled;

			if (scale_to_mv)
			{
				scaled = raw * 2500 / max_adc_value;
			}
			else
			{
				scaled = raw;
			}
			return scaled;
		}

		/****************************************************************************
		*
		* mv_to_adc
		*
		* Convert a millivolt value into an 10-bit ADC count
		*
		*  (useful for setting trigger thresholds)
		*
		****************************************************************************/
		static ushort mv_to_adc(int mv)
		{
			return (ushort)(mv * max_adc_value / 2500);
		}

		/****************************************************************************
		 *
		 * Collect_block_immediate
		 *  this function demonstrates how to collect a single block of data
		 *  from the unit (start collecting immediately)
		 *
		 ****************************************************************************/
		static void collect_block_immediate()
		{
			uint i;
			short[] channels;
			uint nSamples = 1000;
			short nChannels = 1;
			uint nSamplesPerChannel = 1000;
			uint nSamplesCollected;
			ushort[] samples = new ushort[1000];
			uint usForBlock = 1000000;
			ushort[] overflow = new ushort[1000];
			uint triggerIndex = 0;
			StreamWriter fp;

			Console.WriteLine("Collect block immediate...");
			Console.WriteLine("Press a key to start");
			Console.ReadKey();

			channels = new short[1];
			channels[0] = (short)Pico.PL1000_INPUTS.PL1000_CHANNEL_1;

			//Set the trigger (disabled)
			status = (Pico.StatusEnum)Pico.pl1000SetTrigger(g_handle, 0, 0, 0, 0, 0, 0, 0, 0);

			//set sampling rate and channels
			status = (Pico.StatusEnum)Pico.pl1000SetInterval(g_handle, ref usForBlock, nSamples, channels, nChannels);

			fp = new StreamWriter("test.out");

			while (!Console.KeyAvailable)
			{
				//Run
				status = (Pico.StatusEnum)Pico.pl1000Run(g_handle, nSamples, Pico.BLOCK_METHOD.BM_SINGLE);

				//Wait until unit is ready
				short isReady = 0;
				while (isReady == 0)
				{
					status = (Pico.StatusEnum)Pico.pl1000Ready(g_handle, ref isReady);
				}

				nSamplesCollected = nSamplesPerChannel;
				status = (Pico.StatusEnum)Pico.pl1000GetValues(g_handle, samples, ref nSamplesCollected, overflow, ref triggerIndex);

				//Print out the first 10 readings, converting the readings to mV if required
				Console.Clear();
				Console.WriteLine(DateTime.Now.ToString());
				Console.WriteLine("First 10 readings of {0} (press any key to stop)", nSamplesCollected);
				for (i = 0; i < 10; i++)
				{
					Console.WriteLine("{0}", adc_to_mv(samples[i]));
				}
				for (i = 0; i < nSamplesCollected; i++)
				{
					fp.WriteLine("{0}", adc_to_mv(samples[i]));
				}
				Thread.Sleep(100);
			}
			fp.Close();
			status = (Pico.StatusEnum)Pico.pl1000Stop(g_handle);
			Console.ReadKey();
		}

		/****************************************************************************
		 *
		 * Collect_block_triggered
		 *  this function demonstrates how to collect a single block of data from the
		 *  unit, when a trigger event occurs.
		 *
		 ****************************************************************************/
		static void collect_block_triggered()
		{
			uint i;
			short[] channels;
			ushort nSamples = 1000;
			short nChannels = 1;
			ushort nSamplesPerChannel = 1000;
			uint nSamplesCollected;
			ushort[] samples = new ushort[1000];
			uint usForBlock = 1000000;
			ushort[] overflow = new ushort[1000]; // oops
			uint triggerIndex = 0;
			StreamWriter fp;

			Console.WriteLine("Collect block triggered...");
			Console.WriteLine("Collects when value rises past 1 V");
			Console.WriteLine("Press a key to start...");
			Console.ReadKey();

			channels = new short[1];
			channels[0] = (short)Pico.PL1000_INPUTS.PL1000_CHANNEL_1;

			//Set the trigger
			status = (Pico.StatusEnum)Pico.pl1000SetTrigger(g_handle, 1, 0, 0, (ushort)Pico.PL1000_INPUTS.PL1000_CHANNEL_1, 0, mv_to_adc(1000), 0, -50);

			//set sampling rate and channels
			status = (Pico.StatusEnum)Pico.pl1000SetInterval(g_handle, ref usForBlock, nSamples, channels, nChannels);

			Console.WriteLine("Trigger delay is set to -50% (trigger event in centre of block)");
			Console.WriteLine("\nWaiting for trigger...\n");
			Console.WriteLine("Press a key to abort");

			fp = new StreamWriter("test.out");

			//Run
			status = (Pico.StatusEnum)Pico.pl1000Run(g_handle, nSamples, Pico.BLOCK_METHOD.BM_SINGLE);

			//Wait until unit is ready
			short isReady = 0;
			while (isReady == 0 && !Console.KeyAvailable)
			{
				status = (Pico.StatusEnum)Pico.pl1000Ready(g_handle, ref isReady);
			}

			nSamplesCollected = nSamplesPerChannel;
			status = (Pico.StatusEnum)Pico.pl1000GetValues(g_handle, samples, ref nSamplesCollected, overflow, ref triggerIndex);

			//Print out the first 10 readings, converting the readings to mV if required
			Console.WriteLine("5 readings either side of trigger event ({0} samples collected)\n", nSamplesCollected);
			for (i = triggerIndex - 5; i < triggerIndex + 6; i++)
			{
				Console.WriteLine("%d", adc_to_mv(samples[i]));
			}
			for (i = 0; i < nSamplesCollected; i++)
			{
				fp.WriteLine("{0}", adc_to_mv(samples[i]));
			}

			fp.Close();
			status = (Pico.StatusEnum)Pico.pl1000Stop(g_handle);
		}

		/****************************************************************************
		 *
		 * Collect_windowed_blocks
		 *
		 * This function demonstrates how to use windowed blocks.
		 *
		 ****************************************************************************/
		static void collect_windowed_blocks()
		{
			uint i;
			short[] channels;
			ushort nSamples = 1000;
			short nChannels = 1;
			uint nSamplesPerChannel = 1000;
			uint nSamplesCollected;
			ushort[] samples = new ushort[1000];
			uint usForBlock = 10000000;	//10 seconds
			ushort[] overflow = new ushort[1000]; // oops
			uint triggerIndex = 0;
			short nLines = 0;
			StreamWriter fp;

			Console.WriteLine("Collect windowed block...");
			Console.WriteLine("First block appears after 10 seconds,");
			Console.WriteLine("then 10 second blocks are collected every second");
			Console.WriteLine("Press a key to start");
			Console.ReadKey();

			channels = new short[1];
			channels[0] = (short)Pico.PL1000_INPUTS.PL1000_CHANNEL_1;

			//Set the trigger (disabled)
			status = (Pico.StatusEnum)Pico.pl1000SetTrigger(g_handle, 0, 0, 0, 0, 0, 0, 0, 0);

			//set sampling rate and channels
			status = (Pico.StatusEnum)Pico.pl1000SetInterval(g_handle, ref usForBlock, nSamples, channels, nChannels);

			//Start streaming
			status = (Pico.StatusEnum)Pico.pl1000Run(g_handle, nSamples, Pico.BLOCK_METHOD.BM_WINDOW);

			//Wait until unit is ready
			Console.WriteLine("Waiting for first block...");
			short isReady = 0;
			while (isReady == 0)
			{
				status = (Pico.StatusEnum)Pico.pl1000Ready(g_handle, ref isReady);
			}

			Console.WriteLine("Press any key to stop");
			fp = new StreamWriter("test.out");

			while (!Console.KeyAvailable)
			{
				nSamplesCollected = nSamplesPerChannel;
				status = (Pico.StatusEnum)Pico.pl1000GetValues(g_handle, samples, ref nSamplesCollected, overflow, ref triggerIndex);

				Console.WriteLine("{0} values", nSamplesCollected);
				if (nLines == 20)
				{
					Console.Clear();
					Console.WriteLine("Press any key to stop");
					nLines = 0;
				}
				else
				{
					nLines++;
				}

				for (i = 0; i < nSamplesCollected; i++)
				{
					fp.WriteLine("{0}", adc_to_mv(samples[i]));
				}

				Thread.Sleep(1000);		//Wait 1 second before collecting next 10 second block.
			}
			fp.Close();
			status = (Pico.StatusEnum)Pico.pl1000Stop(g_handle);

			Console.ReadKey();
		}


		/****************************************************************************
		 *
		 * Collect_streaming
		 * 
		 * This function demonstrates how to use streaming.
		 *
		 ****************************************************************************/
		static void collect_streaming()
		{
			uint i;
			short[] channels;
			uint nSamples = 1000;
			short nChannels = 1;
			uint nSamplesPerChannel = 1000;
			uint nSamplesCollected;
			ushort[] samples = new ushort[1000];
			uint usForBlock = 1000000;
			ushort[] overflow = new ushort[1000]; // oops
			uint triggerIndex = 0;
			short nLines = 0;

			StreamWriter fp;

			Console.WriteLine("Collect streaming...");
			Console.WriteLine("Data is written to disk file (test.out)");
			Console.WriteLine("Press a key to start");
			Console.ReadKey();

			channels = new short[1];
			channels[0] = (short)Pico.PL1000_INPUTS.PL1000_CHANNEL_1;

			//Set the trigger (disabled)
			status = (Pico.StatusEnum)Pico.pl1000SetTrigger(g_handle, 0, 0, 0, 0, 0, 0, 0, 0);

			//set sampling rate and channels
			status = (Pico.StatusEnum)Pico.pl1000SetInterval(g_handle, ref usForBlock, nSamples, channels, nChannels);

			//Start streaming
			status = (Pico.StatusEnum)Pico.pl1000Run(g_handle, nSamples, Pico.BLOCK_METHOD.BM_STREAM);

			//Wait until unit is ready
			short isReady = 0;
			while (isReady == 0)
			{
				status = (Pico.StatusEnum)Pico.pl1000Ready(g_handle, ref isReady);
			}

			Console.Clear();
			Console.WriteLine("Press any key to stop");

			fp = new StreamWriter("test.out");

			while (!Console.KeyAvailable)
			{
				nSamplesCollected = nSamplesPerChannel;
				status = (Pico.StatusEnum)Pico.pl1000GetValues(g_handle, samples, ref nSamplesCollected, overflow, ref triggerIndex);

				Console.WriteLine("{0} values", nSamplesCollected);

				if (nLines == 20)
				{
					Console.Clear();
					Console.WriteLine("Press any key to stop");
					nLines = 0;
				}
				else
				{
					nLines++;
				}

				for (i = 0; i < nSamplesCollected; i++)
				{
					fp.WriteLine("{0}", adc_to_mv(samples[i]));
				}

				Thread.Sleep(100);
			}
			fp.Close();
			status = (Pico.StatusEnum)Pico.pl1000Stop(g_handle);

			Console.ReadKey();
		}

		/****************************************************************************
		 *
		 * Collect_individual
		 *
		 ****************************************************************************/
		static void collect_individual()
		{
			int sample_no;
			int c;
			short value;

			Console.WriteLine("Collect individual...");
			Console.WriteLine("Takes individual readings under program control");
			Console.WriteLine("Sample from all channels");
			Console.WriteLine("Press a key to start");
			Console.ReadKey();

			sample_no = 20;
			while (!Console.KeyAvailable)
			{
				Thread.Sleep(100);

				if (++sample_no > 20)
				{
					sample_no = 0;
					Console.Clear();
					Console.WriteLine("Press any key to stop");
					for (c = 1; c <= 16; c++)
					{
						Console.Write("c{0:00} ", c);
					}
					Console.WriteLine();
				}
				for (c = 1; c <= 16; c++)
				{
					value = 0;
					Pico.pl1000GetSingle(g_handle, c, ref value);
					Console.Write("{0:000} ", adc_to_mv(value));
				}
				Console.WriteLine();
			}
			Console.ReadKey();
		}

		/****************************************************************************
		 *
		 * Set/clear digital outputs
		 *
		 ****************************************************************************/
		static void outputToggle(Pico.PL1000_DO_CH doChannel)
		{
			switch (doChannel)
			{
				case Pico.PL1000_DO_CH.PL1000_DO_CHANNEL_0:
					if (d0State == 0)
						d0State = 1;
					else
						d0State = 0;

					Pico.pl1000SetDo(g_handle, d0State, (short)doChannel);
					break;
				case Pico.PL1000_DO_CH.PL1000_DO_CHANNEL_1:
					if (d1State == 0)
						d1State = 1;
					else
						d1State = 0;

					Pico.pl1000SetDo(g_handle, d1State, (short)doChannel);
					break;
				case Pico.PL1000_DO_CH.PL1000_DO_CHANNEL_2:
					if (d2State == 0)
						d2State = 1;
					else
						d2State = 0;

					Pico.pl1000SetDo(g_handle, d2State, (short)doChannel);
					break;
				case Pico.PL1000_DO_CH.PL1000_DO_CHANNEL_3:
					if (d3State == 0)
						d3State = 1;
					else
						d3State = 0;

					Pico.pl1000SetDo(g_handle, d3State, (short)doChannel);
					break;
			}
		}

		/****************************************************************************
		 *
		 * Display digital output states
		 *
		 ****************************************************************************/
		static void displayOutputStates()
		{
			Console.WriteLine("\nDigital Outputs");
			Console.WriteLine("===============");
			Console.WriteLine("D0\tD1\tD2\tD3\t");
			Console.WriteLine("{0}\t{1}\t{2}\t{3}\n", d0State, d1State, d2State, d3State);
		}


		/****************************************************************************
		 *
		 * Set pulse width modulation
		 *
		 ****************************************************************************/
		static void pwm()
		{
			int period = 0;
			int cycle = 0;

			Console.WriteLine("\n----------PWM----------\n");
			do
			{
				Console.Write("Enter period (100 to 1800 microseconds):");
				string strPeriode = Console.ReadLine();
				int.TryParse(strPeriode, out period);
			} while (period < 100 || period > 1800);

			do
			{
				Console.Write("Enter duty cycle (0 to 100%):");
				string strCycle = Console.ReadLine();
				int.TryParse(strCycle, out cycle);
			} while (cycle < 0 || cycle > 100);

			Pico.pl1000SetPulseWidth(g_handle, (ushort)period, (byte)cycle);
		}

		static void Main(string[] args)
		{
			Console.WriteLine("pl1000 driver example program");
			Console.WriteLine("Version C# 1.0");
			Console.WriteLine();

			Console.WriteLine("Opening the device...");
			status = (Pico.StatusEnum)Pico.pl1000OpenUnit(ref g_handle);

			if (status != 0)
			{
				Console.WriteLine("Unable to open device:" + (Pico.StatusEnum)status + "\nPress any key");
				Console.ReadKey();
				return;
			}
			else
			{
				Console.WriteLine("Device opened successfully:" + (Pico.StatusEnum)status);
				Console.WriteLine();

				Pico.pl1000SetDo(g_handle, 0, (short)Pico.PL1000_DO_CH.PL1000_DO_CHANNEL_0);
				Pico.pl1000SetDo(g_handle, 0, (short)Pico.PL1000_DO_CH.PL1000_DO_CHANNEL_1);
				Pico.pl1000SetDo(g_handle, 0, (short)Pico.PL1000_DO_CH.PL1000_DO_CHANNEL_2);
				Pico.pl1000SetDo(g_handle, 0, (short)Pico.PL1000_DO_CH.PL1000_DO_CHANNEL_3);

				StringBuilder sb = new StringBuilder(255);
				int size = 0;
				status = (Pico.StatusEnum)Pico.pl1000GetUnitInfo(g_handle, sb, sb.Capacity, ref size, Pico.PICO_VARIANT_INFO);
				Console.WriteLine("Model PicoLog\t\t\t" + sb);

				status = (Pico.StatusEnum)Pico.pl1000GetUnitInfo(g_handle, sb, sb.Capacity, ref size, Pico.PICO_BATCH_AND_SERIAL);
				Console.WriteLine("Serial Number\t\t\t" + sb);

				status = (Pico.StatusEnum)Pico.pl1000GetUnitInfo(g_handle, sb, sb.Capacity, ref size, Pico.PICO_CAL_DATE);
				Console.WriteLine("Calibration Date\t\t\t" + sb);

				status = (Pico.StatusEnum)Pico.pl1000GetUnitInfo(g_handle, sb, sb.Capacity, ref size, Pico.PICO_USB_VERSION);
				Console.WriteLine("USB Version\t\t\t" + sb);

				status = (Pico.StatusEnum)Pico.pl1000GetUnitInfo(g_handle, sb, sb.Capacity, ref size, Pico.PICO_HARDWARE_VERSION);
				Console.WriteLine("Hardware version\t\t\t" + sb);

				status = (Pico.StatusEnum)Pico.pl1000GetUnitInfo(g_handle, sb, sb.Capacity, ref size, Pico.PICO_DRIVER_VERSION);
				Console.WriteLine("pl1000.dll version\t\t\t" + sb);

				status = (Pico.StatusEnum)Pico.pl1000GetUnitInfo(g_handle, sb, sb.Capacity, ref size, Pico.PICO_KERNEL_VERSION);
				Console.WriteLine("Kernel version\t\t\t" + sb);

				status = (Pico.StatusEnum)Pico.pl1000MaxValue(g_handle, ref max_adc_value);

				char ch = ' ';
				while (ch != 'X')
				{
					Console.WriteLine();
					Console.WriteLine("Select an operation");
					Console.WriteLine("B - immediate block\t\tA - toggle ADC/mV");
					Console.WriteLine("T - triggered block\t\tP - set PWM\n");
					Console.WriteLine("W - windowed block\t\tD - display digital output states");
					Console.WriteLine("S - streaming\t\t\t0,1,2,3 - toggle digital output");
					Console.WriteLine("I - individual reading\t\tX - exit");
					Console.Write("Your choise: ");
					ch = Console.ReadKey().KeyChar;
					ch = ch.ToString().ToUpper()[0];
					Console.WriteLine();

					switch (ch)
					{
						case 'X':
							break;

						case 'A':
							scale_to_mv = !scale_to_mv;
							if (scale_to_mv)
								Console.WriteLine("Values will be displayed in mV");
							else
								Console.WriteLine("Values will be displayed in ADC counts");
							break;

						case 'B':
							collect_block_immediate();
							break;

						case 'T':
							collect_block_triggered();
							break;

						case 'W':
							collect_windowed_blocks();
							break;

						case 'S':
							collect_streaming();
							break;

						case 'I':
							collect_individual();
							break;

						case 'P':
							pwm();
							break;

						case 'D':
							displayOutputStates();
							break;

						case '0':
							outputToggle(Pico.PL1000_DO_CH.PL1000_DO_CHANNEL_0);
							displayOutputStates();
							break;

						case '1':
							outputToggle(Pico.PL1000_DO_CH.PL1000_DO_CHANNEL_1);
							displayOutputStates();
							break;

						case '2':
							outputToggle(Pico.PL1000_DO_CH.PL1000_DO_CHANNEL_2);
							displayOutputStates();
							break;

						case '3':
							outputToggle(Pico.PL1000_DO_CH.PL1000_DO_CHANNEL_3);
							displayOutputStates();
							break;
						default:
							Console.WriteLine("Invalid operation\n");
							break;
					}
				}
				Console.WriteLine("Closing device...");
				status = (Pico.StatusEnum)Pico.pl1000CloseUnit(g_handle);
				Console.WriteLine(status);
			}
		}
	}
}
